;;; --------------------------------------------------------------------------;
;;; THREAD.LSP - By David Kitson. 
;;;   (CC) Creative Commons BY-SA 
;;; version 0.1 alpha
;;;  
;;;   A mathematical approximation of a thread profile as an alternate method
;;;   of making a Screw Thread by generating a cross-sectional profile of a 
;;;   thread and SWEEPing it around the axis, with BASE and TWIST - Allows for
;;;   very complicated threads and both single and multi-index threads without
;;;   requiring overrun. An alternative to the traditional method of simulating
;;;   a lathe and cutting a thread with a triangle around a spiral.
;;;   
;;;   VERY MUCH IMPROVED OVER THE OLD METHOD... At least I hope it is... 
;;;
;;;   Inspired by the AutocAD SPIRAL program, so written in that style. 
;;;
;;; --------------------------------------------------------------------------;
;;; DESCRIPTION
;;;
;;;   This is based on a programming example.
;;;
;;;   Designed and implemented by David Grant Kitson in December 2013 when he
;;;	discovered that CorelCAD had a faulty SWEEP function and couldn't cut
;;; 	threads in the traditional way. It also had a faulty JOIN ( WELD )
;;;	function, and this couldn't be done manually... Yes, writing LISP 
;;;	without experience is a pretty crazy idea. So I examined how others 
;;;	had done it before me.
;;;
;;;   This program constructs a THREAD PROFILE. It can be loaded and called 
;;;   by typing either "thread" or the following:
;;;   (cthread <base point> <diameter> <pitch>).
;;;
;;;   It currently assumed 32 segments per "circle" - because it divides up
;;;	nicely.
;;;
;;; --------------------------------------------------------------------------;

(defun myerror (s)                    ; If an error (such as CTRL-C) occurs
                                      ; while this command is active...
  (if (/= s "Function cancelled")
    (princ (strcat "\nError: " s))
  )
  (setvar "cmdecho" ocmd)             ; Restore saved modes
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
)


;;; Inside thread first... Because I don't really know what I'm doing yet.
;;; So later will call other names - eg, othread, mithread, mothread
;;; Should produce an ISO conforming standard metric thread.

(defun connect ( sections )
  (command "_loft" sections)
  )

(defun boltaxis (start end)
  (command "_polyline3d" start)
  (command end)
  (command "")
  )

(defun cbolt (basepoint diameter pitch / segments rads rinc id segmentangle faceinc facedist nextpoint startpoint) 
  (setvar "blipmode" 0)                         ; turn blipmode off
  (setvar "cmdecho" 0)                          ; turn cmdecho off
  (setq segments 32)		                ; 32 segments in this circle.
  (setq rads (* 3.141592653 2))                 ; PI x 2 ( Radians in a circle )
  (setq rinc (/ rads segments))                 ; Angle Increment in Radians = 1/Segment of a circle.
  (setq id (- diameter (* 0.75775 pitch)))       ; Internal Diameter = 7/8 of Thread Height ( ISO standard )
  (setq segmentangle 0.0)	                ; Segmentangle = 0 ( starting at origin... )
  (setq faceinc (*  pitch (/ 1.732 segments)))  ; 60 degrees step per variation. 
  (setq facedist (/ diameter 2))	        ; initial distance of face from center = radius.
  (setq startpoint (polar basepoint 0 facedist)) ; Set the last point to the same as the first point. 
  (command "_pline" startpoint)                  ; start of thread is also the same place the thread ends... Start the line.



  (repeat 4 					; Draw the top of the thread... The DIAMETER of the BOLT or the M number...
      (setq segmentangle ( + segmentangle rinc ))
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  ) 						; This should produce a 1/4 circle... With 8 segments.


  (repeat 12 					; Draw the first angular face... Approx 60 degrees.
      (setq segmentangle ( + segmentangle rinc ))
      (setq facedist (- facedist faceinc))	; Should approximate 60 degree ISO face when SWEEP is run. 
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  ) 						; This should produce a 1/4 circle... With 8 segments.

 (repeat 4 					; Draw the bottom of the thread, with spare space...
      (setq segmentangle ( + segmentangle rinc ))
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  )

  (repeat 11 					; Draw the other angular face, the far thread wall. 60 degrees.
      (setq segmentangle ( + segmentangle rinc ))
      (setq facedist (+ facedist faceinc))	; Now we go back the other way for the other face. 
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  ) 						; This should produce a 1/4 circle... With 8 segments.

  (command "C")					; Close? Yep, Close...  
;;;  (command "")                                  ; No more points to define.. How do I close it?
  (princ)
) 

(defun cnut (basepoint diameter pitch / segments rads rinc id segmentangle faceinc facedist nextpoint startpoint) 
  (setvar "blipmode" 0)                         ; turn blipmode off
  (setvar "cmdecho" 0)                          ; turn cmdecho off
  (setq segments 32)		                ; 32 segments in this circle.
  (setq rads (* 3.141592653 2))                 ; PI x 2 ( Radians in a circle )
  (setq rinc (/ rads segments))                 ; Angle Increment in Radians = 1/Segment of a circle.
  (setq segmentangle 0.0)	                ; Segmentangle = 0 ( starting at origin... )
  (setq faceinc (*  pitch (/ 1.732 segments)))  ; difference in face distance per segment ( contact face )
  (setq diameter (+ diameter (* 0.054125 pitch))); Adjust outside "starting" diameter to allow for nut thread start diameter
  (setq id (- diameter (* 0.6495 pitch)))       ; Internal Diameter = 3/4 of Thread Height ( ISO standard )
  (setq facedist (/ diameter 2))	        ; initial distance of face from center = radius.
  (setq startpoint (polar basepoint 0 facedist)) ; Set the last point to the same as the first point. 
  (command "_pline" startpoint)                  ; start of thread is also the same place the thread ends... Start the line.



  (repeat 2 					; Draw the top of the thread... The DIAMETER of the BOLT or the M number...
      (setq segmentangle ( + segmentangle rinc ))
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  ) 						; This should produce a 1/4 circle... With 8 segments.


  (repeat 11 					; Draw the first angular face... Approx 60 degrees.
      (setq segmentangle ( + segmentangle rinc ))
      (setq facedist (- facedist faceinc))	; Should approximate 60 degree ISO face when SWEEP is run. 
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  ) 						; This should produce a 1/4 circle... With 8 segments.

 (repeat 8 					; Draw the bottom of the thread, with spare space...
      (setq segmentangle ( + segmentangle rinc ))
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  )

  (repeat 10 					; Draw the other angular face, the far thread wall. 60 degrees.
      (setq segmentangle ( + segmentangle rinc ))
      (setq facedist (+ facedist faceinc))	; Now we go back the other way for the other face. 
      (setq nextpoint (polar basepoint 
                      segmentangle
                      facedist
               ))
      (command nextpoint)                              ; Give it ( base angle distance ) - for each point.
  ) 						; This should produce a 1/4 circle... With 8 segments.

  (command "C")					; Close? Yep, Close...  
;;;  (command "")                                  ; No more points to define.. How do I close it?
  (princ)
) 

;;;
;;;       Interactive thread generation
;;;



(defun C:BOLT (/ olderr ocmd oblp basep od pitch length twistlen boltend boltface baxis allparts step nextstep allsteps) 
  (setq olderr  *error*
        *error* myerror)
  (setq ocmd (getvar "cmdecho"))
  (setq oblp (getvar "blipmode"))
  (setvar "cmdecho" 0)
  (initget 1)                         ; Can't just press enter. 
  (setq basep (getpoint "\nCenter point: "))
  (initget 7)                         ; OD must not be null, zero or negative.
  (setq od (getdist "\nDiameter: "))
  (initget 7)                         ; pitch must not be null, zero or negative
  (setq pitch (getdist "\nPitch: "))
  (initget 7)                         ; length must not be null, zero or negative
  (setq length (getdist "\nLength: "))

  (cbolt basep od pitch)
  (setq boltend (list (car basep) (cadr basep) (+ (caddr basep) length)))
  (boltaxis basep boltend)

  (setvar "cmdecho" ocmd)
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
)

(defun C:NUT (/ olderr ocmd oblp basep od pitch length twistlen boltend boltface baxis allparts step nextstep allsteps clearance ) 
  (setq olderr  *error*
        *error* myerror)
  (setq ocmd (getvar "cmdecho"))
  (setq oblp (getvar "blipmode"))
  (setvar "cmdecho" 0)
  (initget 1)                         ; Can't just press enter. 
  (setq basep (getpoint "\nCenter point: "))
  (initget 7)                         ; OD must not be null, zero or negative.
  (setq od (getdist "\nDiameter: "))
  (initget 7)                         ; pitch must not be null, zero or negative
  (setq pitch (getdist "\nPitch: "))
  (initget 7)                         ; length must not be null, zero or negative
  (setq length (getdist "\nLength: "))

  (initget 4)                         ; Diameter clearance must not be zero or negative
  (setq clearance (getdist "\nClearance between parts to allow for 3D printing shrinkage (0.0): "))
  (cond ((null clearance) (setq clearance 0.0))) 

  (setq clearance (* clearance 2))
  (setq od (+ od clearance ))

  (cnut basep od pitch)
  (setq boltend (list (car basep) (cadr basep) (+ (caddr basep) length)))
  (boltaxis basep boltend)

  (setvar "cmdecho" ocmd)
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)

)

;;; --------------------------------------------------------------------------;

